from turbogears import widgets

class GoogleChart(widgets.Widget):
    """A simple widget to display Google Chart image links.

    Example usage::

        widget = GoogleChart(attrs={'class': 'googlechart'},
            width=500, height=200)
        widget.display((('Python', 60), ('Perl', 15), ('PHP', 25)))

    or::

        widget.display(dict(Python=60, Perl=15, PHP=25), type='bvs')

    or::

        widget.display([60, 15, 25], labels=['Python', 'Perl', 'PHP'],
           type='bhs')

    The widget value is taken as the chart data. It can be either a simple list
    or tuple of values, a list or tuple of (label, value) items or a dictionary
    with label=value items.

    """
    template="""\
    <img xmlns:py="http://purl.org/kid/ns#"
      src="${api_base_uri}?cht=$type&amp;chs=$size&amp;chl=$labels&amp;chd=$data"
      width="$width"
      height="$height"
      py:attrs="attrs"
    />
    """
    params = ['api_base_uri', 'attrs', 'type', 'width', 'height', 'labels']
    params_doc = dict(
        attrs = 'Dictionary containing extra (X)HTML attributes for the IMG tag',
        api_base_uri = 'The base URI of the Google Chart web service',
        type = "The code for the chart type (default: 'p3' (a pie chart))",
        width = "The width of teh chart image in pixels",
        height = "The height of the chart image in pixels",
        labels = "The labels for the chart data values. Can be a tuple or list"
            " of values or a string of values separated by pipe symbols ('|')"
    )

    api_base_uri = 'http://chart.apis.google.com/chart'
    attrs = {}
    height = 150
    labels = []
    type = 'p3'
    width = 250

    def update_params(self, params):
        super(GoogleChart, self).update_params(params)
        value = params.get('value', [])
        labels = params.get('labels', [])
        if isinstance(value, dict):
            # value is a dict of label=value items
            value = value.items()
        if isinstance(value, (list, tuple)):
            if value and isinstance(value[0], (list, tuple)):
                # value is a list of (label, value) items
                params['labels'] = "|".join([i[0] for i in value])
                params['data'] = "t:" + ",".join([str(i[1]) for i in value])
            else:
                # value is a list of values
                params['data'] = "t:" + ",".join([str(i) for i in value])
        else:
            params['data'] = value
        # explicit 'labels' keyword arg overrides labels in data list or dict
        if labels and isinstance(labels, (list, tuple)):
            params['labels'] = "|".join(list(labels))
        params['size'] = "%sx%s" % (params['width'], params['height'])
